/*
 * Copyright (c) 2001-2006, John Mettraux, OpenWFE.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * . Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.  
 * 
 * . Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution.
 * 
 * . Neither the name of the "OpenWFE" nor the names of its contributors may be
 *   used to endorse or promote products derived from this software without
 *   specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: WorkListAction.java 2494 2006-04-15 21:15:08Z jmettraux $
 */

//
// WorkListAction.java
//
// jmettraux@openwfe.org
//
// generated with 
// jtmpl 1.0.04 31.10.2002 John Mettraux (jmettraux@openwfe.org)
//

package openwfe.org.wlactions;

import javax.security.auth.Subject;
import openwfe.org.ApplicationContext;
import openwfe.org.worklist.Definitions;
import openwfe.org.worklist.store.WorkItemStore;
import openwfe.org.worklist.store.StoreException;


/**
 * The basic worklist action. Useful because it contains fields necessary
 * for running the desired actions.
 *
 * <p><font size=2>CVS Info :
 * <br>$Author: jmettraux $
 * <br>$Date: 2006-04-15 23:15:08 +0200 (Sat, 15 Apr 2006) $
 * <br>$Id: WorkListAction.java 2494 2006-04-15 21:15:08Z jmettraux $ </font>
 *
 * @author john.mettraux@openwfe.org
 */
public abstract class WorkListAction

    implements java.security.PrivilegedExceptionAction

{

    //
    // FIELDS

    private ApplicationContext worklistContext = null;
    private Subject subject = null;
    private String storeName = null;

    //
    // CONSTRUCTORS

    public WorkListAction 
        (final ApplicationContext worklistContext,
         final Subject subject,
         final String storeName)
    throws 
        StoreException
    {
        this.worklistContext = worklistContext;
        this.subject = subject;
        this.storeName = storeName;
        
        //
        // a small control
        
        if (getWorkItemStore() == null)
        {
            throw new StoreException
                ("There is no store named '"+this.storeName+"'");
        }
    }

    //
    // METHODS

    public ApplicationContext getContext () { return this.worklistContext; }

    public Subject getSubject () { return this.subject; }
    public String getStoreName () { return this.storeName; }

    public WorkItemStore getWorkItemStore ()
    {
        //return (WorkItemStore)this.worklistContext
        //    .get(Definitions.STORE_PREFIX + this.storeName);

        return (WorkItemStore)this.worklistContext.lookup(this.storeName);
    }

}
